# Standard Library
from typing import List


class VcsInstance:
    """
    A class to represent VCS instance.
    Attributes
    ----------
    provider_type : str
        vcs provider type
    scheme : str
        scheme of vcs instance
    host : str
        host of vcs instance
    port : str
        port of vcs instance
    username : str
        username of vcs instance
    password : str
        password of vcs instance
    organization : str
        organization of vcs instance
    scope : list
        List of scope
    """

    def __init__(self,
                 provider_type: str,
                 scheme: str,
                 host: str,
                 port: str,
                 username: str,
                 password: str,
                 organization: str,
                 scope: List[str]):
        self.provider_type: str = provider_type
        self.scheme: str = scheme
        self.host: str = host
        self.port: str = port
        self.username: str = username
        self.password: str = password
        self.organization: str = organization
        self.scope: list = scope



# Standard Library
import logging
import sys

# First Party
from resc_helm_wizard import common, questions
from resc_helm_wizard.helm_value import HelmValue


def prompt_questions():
    """
        prompt set of questions to user in order to generate values yaml file for helm deployment of RESC
    :raises KeyboardInterrupt: if there is any keyboard interruption from user
    """
    try:
        os_input = questions.ask_operating_system()
        if os_input:
            operating_system = common.get_operating_system(user_input=os_input)

        if operating_system:
            storage_path = common.create_storage_for_db_and_rabbitmq(operating_system=operating_system)
            db_storage_path = storage_path["db_storage_path"]
            rabbitmq_storage_path = storage_path["rabbitmq_storage_path"]

            db_password = questions.ask_password_for_database()

            vcs_instances = common.get_vcs_instance_question_answers()

            if (db_password and db_storage_path and
                    rabbitmq_storage_path and vcs_instances):
                helm_values = HelmValue(
                    operating_system=operating_system,
                    db_password=db_password,
                    db_storage_path=db_storage_path,
                    rabbitmq_storage_path=rabbitmq_storage_path,
                    vcs_instances=vcs_instances
                )

                common.create_helm_values_yaml(helm_values=helm_values,
                                               input_values_yaml_file="resources/example-values.yaml")
                common.run_deployment_as_per_user_confirmation()
    except KeyboardInterrupt:
        logging.error("Aborting the program! operation cancelled by user")
        sys.exit(-1)


if __name__ == '__main__':
    prompt_questions()



# Standard Library
import json
import logging
import subprocess
import sys

# First Party
from resc_helm_wizard import constants

logging.basicConfig(level=logging.INFO)


def install_or_upgrade_helm_release(action: str) -> bool:
    """
        Install or upgrade a helm release
    :param action:
        action to perform like install or upgrade
    :return: bool
        Returns true if install or upgrade succeeded else returns false
    """
    logging.info(f"Running {action}. Please wait for a moment...")
    helm_command = ["helm", action, "-n", constants.NAMESPACE, constants.RELEASE_NAME, constants.CHART_NAME, "-f",
                    constants.VALUES_FILE, "--set-file", "global.secretScanRulePackConfig=" + constants.RULE_FILE,
                    "--repo", constants.RESC_HELM_REPO_URL]
    try:
        output = subprocess.check_output(helm_command)
        logging.info(output.decode("utf-8"))
        return True
    except subprocess.CalledProcessError:
        logging.error(f"An error occurred during {constants.CHART_NAME} deployment")
        return False


def is_chart_version_already_installed() -> bool:
    """
        Checks if chart installed or not
    :return: bool
        Returns true if chart already installed else returns false
    """
    downloaded_version = get_version_from_downloaded_chart()
    installed_version = get_version_from_installed_chart()
    return downloaded_version == installed_version


def get_version_from_installed_chart() -> str:
    """
        Get version of the installed chart
    :return: str
        Returns version of the installed chart
    """
    cmd = f"helm list -f {constants.CHART_NAME} -n {constants.NAMESPACE} -o json"
    output = subprocess.check_output(cmd, shell=True)
    chart_info = json.loads(output.decode("utf-8"))
    if chart_info and "chart" in chart_info[0]:
        return chart_info[0]["chart"].split("-")[1]
    return None


def get_version_from_downloaded_chart() -> str:
    """
        Get version of the downloaded chart
    :return: str
        Returns version of the downloaded chart
    """
    cmd = f"helm search repo {constants.HELM_REPO_NAME} -n {constants.NAMESPACE} -o json"
    output = subprocess.check_output(cmd, shell=True)
    chart_info = json.loads(output.decode("utf-8"))

    if chart_info:
        return chart_info[0]["version"]
    return None


def add_helm_repository():
    """
        Adds a helm repository
    """
    cmd = ["helm", "repo", "add", constants.HELM_REPO_NAME, constants.RESC_HELM_REPO_URL]
    try:
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError:
        logging.error("An error occurred while adding the helm repository")
        sys.exit(1)


def update_helm_repository():
    """
        Updates a helm repository
    """
    cmd = ["helm", "repo", "update", constants.HELM_REPO_NAME]
    try:
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError:
        logging.error("An error occurred while updating the helm repository")
        sys.exit(1)



# Standard Library
import logging
import subprocess

logging.basicConfig(level=logging.INFO)


def create_namespace_if_not_exists(namespace_name: str) -> bool:
    """
        Create a namespace if not exists
    :param namespace_name:
        name of the namespace you want to create
    :return: bool
        Returns true if namespace created else returns false
    """
    created = False
    # Check if the namespace already exists
    check_namespace = subprocess.run(["kubectl", "get", "namespace", namespace_name], capture_output=True, text=True,
                                     check=False)
    if "NotFound" in check_namespace.stderr:
        # Namespace doesn't exist, create it
        create_namespace = subprocess.run(["kubectl", "create", "namespace", namespace_name], check=True)
        if create_namespace.returncode == 0:
            created = True
            logging.info(f"Namespace {namespace_name} created. Preparing for deployment...")
        else:
            logging.error(f"Error reading namespace: {namespace_name}. Aborting deployment...")
    else:
        created = True
        logging.info(f"Namespace {namespace_name} already exists. Preparing for deployment...")
    return created



# Standard Library
import re


def password_validator(password: str):
    """
        Password validator for database
    :param password:
        password which needs to be validated
    :return: str or bool.
        If validation fails, the output will contain a validation error message.
        Otherwise, the output will return true if validation was successful
    """
    regex = re.compile(r"^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*#?&])[A-Za-z\d@$!#%*?&]{8,128}$")

    if not re.fullmatch(regex, password):
        return "Password must contain at least one upper case, one lower case, one number, " \
               "one special character and the length of the password to be between 8 and 128"
    return True


def github_token_validator(token: str):
    """
        Personal access token validator for GitHub
    :param token:
        token which needs to be validated
    :return: str or bool.
        If validation fails, the output will contain a validation error message.
        Otherwise, the output will return true if validation was successful
    """
    classic_pat_regex = re.compile(r"^ghp_[a-zA-Z0-9]{36}$")
    fine_grained_pat_regex = re.compile(r"^github_pat_[a-zA-Z0-9]{22}_[a-zA-Z0-9]{59}$")

    if not re.fullmatch(classic_pat_regex, token) and not re.fullmatch(fine_grained_pat_regex, token):
        return "Validation failed for provided GitHub token"
    return True


def azure_devops_token_validator(token: str):
    """
        Personal access token validator for Azure Devops
    :param token:
        token which needs to be validated
    :return: str or bool.
        If validation fails, the output will contain a validation error message.
        Otherwise, the output will return true if validation was successful
    """
    regex = re.compile(r"^[a-z0-9]{52}$")

    if not re.fullmatch(regex, token):
        return "Validation failed for provided Azure DevOps token"
    return True


def bitbucket_token_validator(token):
    """
        Personal access token validator for Bitbucket
    :param token:
        token which needs to be validated
    :return: str or bool.
        If validation fails, the output will contain a validation error message.
        Otherwise, the output will return true if validation was successful
    """
    regex = re.compile(r"^(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z]).{40,50}$")

    if not re.fullmatch(regex, token):
        return "Validation failed for provided Bitbucket token"
    return True


def github_account_name_validator(github_accounts):
    """
        GitHub account name validator
    :param github_accounts:
        comma separated list of GitHub accounts
    :return: str or bool.
        If validation fails, the output will contain a validation error message.
        Otherwise, the output will return true if validation was successful
    """
    input_list = [elem.strip() for elem in github_accounts.split(",")]
    regex = re.compile(r"^[a-zA-Z\d](?:[a-zA-Z\d]|-(?=[a-zA-Z\d])){0,38}$")
    for account in input_list:
        if not re.fullmatch(regex, account):
            if account:
                msg = f"{account} is not a valid GitHub account"
            else:
                msg = "Please enter a valid comma separated list of GitHub accounts you want to scan"
            return msg
    return True



# Standard Library
from typing import List

# First Party
from resc_helm_wizard.vcs_instance import VcsInstance


class HelmValue:
    """
    A class to represent user provided Helm values.
    Attributes
    ----------
    operating_system : str
        Operating system
    db_password : str
        Database password
    db_storage_path : str
        Database storage path
    rabbitmq_storage_path : str
        Rabbitmq storage path
    vcs_instances : list
        List of VCS instances
    """

    def __init__(self,
                 operating_system: str,
                 db_password: str,
                 db_storage_path: str,
                 rabbitmq_storage_path: str,
                 vcs_instances: List[VcsInstance]):
        self.operating_system: str = operating_system
        self.db_password: str = db_password
        self.db_storage_path: str = db_storage_path
        self.rabbitmq_storage_path: str = rabbitmq_storage_path
        self.vcs_instances: list = vcs_instances



RULE_FILE = "RESC-RULE.toml"
RULE_FILE_URL = "https://raw.githubusercontent.com/zricethezav/gitleaks/master/config/gitleaks.toml"
NAMESPACE = "resc"
RESC_HELM_REPO_URL = "https://abnamro.github.io/repository-scanner"
HELM_REPO_NAME = "resc-helm-repo"
RELEASE_NAME = "resc"
CHART_NAME = "resc"
VALUES_FILE = "custom-values.yaml"



# Standard Library
import os

# Third Party
import questionary

# First Party
from resc_helm_wizard.validator import (
    azure_devops_token_validator,
    bitbucket_token_validator,
    github_account_name_validator,
    github_token_validator,
    password_validator
)


def ask_operating_system() -> str:
    """
        Asks user to select operating system
    :return: str
        Returns user selected operating system
    """
    answer = questionary.select(
        message="Which operating system are you running on the target environment",
        choices=["Microsoft Windows", "macOS", "Linux"]).unsafe_ask()
    return answer


def ask_local_storage_path() -> str:
    """
        Asks user to provide path for local storage
    :return: str
        Returns user provided local storage path
    """
    default_local_storage = os.path.expanduser('~')
    answer = questionary.path(message="Where would you like to create the local storage for RESC. default is ",
                              default=default_local_storage, only_directories=True).unsafe_ask()
    return answer


def ask_password_for_database() -> str:
    """
        Asks user to provide password for database
    :return: str
        Returns user provided password for database
    """
    answer = questionary.password("Please enter the password you want to set for database",
                                  validate=password_validator).unsafe_ask()
    return answer


def ask_user_confirmation(msg: str) -> bool:
    """
        Asks user to provide confirmation
    :param msg:
        confirmation message
    :return: bool
        Returns True or False based on user's confirmation
    """
    answer = questionary.confirm(msg).unsafe_ask()
    return answer


def ask_user_to_select_vcs_instance() -> [str]:
    """
        Asks user to select vcs instances
    :return: [str]
        Returns array of user selected vcs instances
    """
    answer = questionary.checkbox(
        'Select VCS instance for which you want to run the scan',
        choices=[
            "GitHub",
            "Azure Devops",
            "Bitbucket",
        ],
        default="GitHub").unsafe_ask()
    return answer


def ask_vcs_instance_details(vcs_type: str) -> dict:
    """
        Asks user to provide vcs instances details
    :return: dict
        Returns vcs instance info
    """
    username = "NA"
    organization = ""
    url = questionary.text(f"Please enter {vcs_type} url").unsafe_ask()

    if vcs_type == "GitHub":
        username = questionary.text(f"What's your {vcs_type} username").unsafe_ask()
        token = questionary.password(f"Please enter your {vcs_type} personal access token",
                                     validate=github_token_validator).unsafe_ask()

    if vcs_type == "Bitbucket":
        username = questionary.text(f"What's your {vcs_type} username").unsafe_ask()
        token = questionary.password(f"Please enter your {vcs_type} personal access token",
                                     validate=bitbucket_token_validator).unsafe_ask()

    if vcs_type == "Azure Devops":
        organization = questionary.text(f"What's your organization name in {vcs_type}").unsafe_ask()
        token = questionary.password(f"Please enter your {vcs_type} personal access token",
                                     validate=azure_devops_token_validator).unsafe_ask()
    vcs_instance_info = {"url": url, "organization": organization, "username": username, "token": token}
    return vcs_instance_info


def ask_which_github_accounts_to_scan(default_github_accounts: str) -> [str]:
    """
        Asks user to provide GitHub account names to scan
    :return: [str]
        Returns array of GitHub account names
    """
    github_accounts = questionary.text("Enter a comma separated list of GitHub accounts you want to scan",
                                       default=default_github_accounts,
                                       validate=github_account_name_validator).unsafe_ask()
    return github_accounts



# Standard Library
import logging
import os
import subprocess
import sys
from typing import List
from urllib.parse import urlparse

# Third Party
import requests
import yaml

# First Party
from resc_helm_wizard import constants, questions
from resc_helm_wizard.helm_utilities import (
    add_helm_repository,
    install_or_upgrade_helm_release,
    is_chart_version_already_installed,
    update_helm_repository
)
from resc_helm_wizard.helm_value import HelmValue
from resc_helm_wizard.kubernetes_utilities import create_namespace_if_not_exists
from resc_helm_wizard.vcs_instance import VcsInstance

logging.basicConfig(level=logging.INFO)


def get_operating_system(user_input: str) -> str:
    """
        Retrieve operating system
    :param user_input:
        input from user
    :return: str
        Returns windows or linux based on the user input
    """
    if user_input == "Microsoft Windows":
        operating_system = "windows"
    else:
        operating_system = "linux"
    return operating_system


def create_storage_for_db_and_rabbitmq(operating_system: str) -> dict:
    """
        Creates volume storage for database and rabbitmq
    :param operating_system:
        operating system
    :return: dict
        Returns dictionary containing database storage and rabbitmq storage path
    """
    local_storage = questions.ask_local_storage_path()

    if os.path.exists(local_storage):
        db_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage, tool_type="database",
                                            create_dir=True)
        rabbitmq_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage,
                                                  tool_type="rabbitmq", create_dir=True)
    else:
        dir_confirm_msg = f"Do you want to create the directory {local_storage}?"
        dir_confirm = questions.ask_user_confirmation(msg=dir_confirm_msg)
        if dir_confirm is True:
            db_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage,
                                                tool_type="database", create_dir=True)
            rabbitmq_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage,
                                                      tool_type="rabbitmq", create_dir=True)
        else:
            logging.warning(
                "Warning! Please ensure the provided directory exists on the system where you are running the "
                "deployment")
            proceed_confirm = questions.ask_user_confirmation(msg="Are you sure you want to proceed?")
            if proceed_confirm is True:
                db_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage,
                                                    tool_type="database", create_dir=False)
                rabbitmq_storage_path = generate_pvc_path(operating_system=operating_system, path=local_storage,
                                                          tool_type="rabbitmq", create_dir=False)
            else:
                logging.info("Aborting the program!!")
                sys.exit(1)
    storage_path = {"db_storage_path": db_storage_path, "rabbitmq_storage_path": rabbitmq_storage_path}
    return storage_path


def generate_pvc_path(operating_system: str, path: str, tool_type: str, create_dir: bool) -> str:
    """
        Generates volume claim path for database and rabbitmq
    :param operating_system:
        operating system
    :param path:
        path provided by user
    :param tool_type:
        tool type either database or rabbitmq
    :param create_dir:
        should create directory or not
    :return: str
        Returns volume claim path
    """
    if tool_type == "database":
        path = os.path.join(path, "resc-db-storage")
    if tool_type == "rabbitmq":
        path = os.path.join(path, "resc-rabbitmq-storage")
    if create_dir:
        if not os.path.exists(path):
            os.makedirs(path)
            logging.info(f"Storage created for {tool_type} at {path}")
        else:
            logging.info(f"Path already exists. Going to use {path} for {tool_type} storage")

    if operating_system == "windows":
        pvc_path = path.replace(path.split(':')[0], path.split(':')[0].lower())
        pvc_path = pvc_path.replace('\\', '/')
        pvc_path = pvc_path.replace(':', '')
        pvc_path = f"/run/desktop/mnt/host/{pvc_path}"
    else:
        pvc_path = path
    return pvc_path


def prepare_vcs_instances_for_helm_values(helm_values: HelmValue) -> List[VcsInstance]:
    """
        Prepares vcs instances list for helm
    :param helm_values:
        object of HelmValue
    :return: List[VcsInstance]
        Returns list of VCS instances
    """
    vcs_instances: List[VcsInstance] = []
    for vcs in helm_values.vcs_instances:
        if vcs.provider_type == "GITHUB_PUBLIC":
            user_name = "GITHUB_PUBLIC_USERNAME"
            token = "GITHUB_PUBLIC_TOKEN"
        if vcs.provider_type == "AZURE_DEVOPS":
            user_name = "AZURE_DEVOPS_USERNAME"
            token = "AZURE_DEVOPS_TOKEN"
        if vcs.provider_type == "BITBUCKET":
            user_name = "BITBUCKET_USERNAME"
            token = "BITBUCKET_TOKEN"
        vcs_instance_obj = {"name": vcs.provider_type, "scope": vcs.scope, "exceptions": [],
                            "providerType": vcs.provider_type, "hostname": vcs.host, "port": vcs.port,
                            "scheme": vcs.scheme, "username": user_name, "usernameValue": vcs.username,
                            "organization": vcs.organization, "token": token, "tokenValue": vcs.password}
        vcs_instances.append(vcs_instance_obj)
    return vcs_instances


def create_helm_values_yaml(helm_values: HelmValue, input_values_yaml_file: str) -> bool:
    """
        Generates values yaml file for helm deployment of resc
    :param helm_values:
        object of HelmValue
    :param input_values_yaml_file:
        input values.yaml_file path
    :return: bool
        Returns True if file created else returns false
    :raises FileNotFoundError: if example-values.yaml file was not found
    :raises KeyError: if any expected key was not found in the values dictionary
    """
    output_file_generated = False
    output_values_yaml_file = constants.VALUES_FILE
    helm_deployment_help_link = "https://github.com/abnamro/repository-scanner/" \
                                "blob/main/deployment/kubernetes/README.md"

    try:
        with open(input_values_yaml_file, "r", encoding="utf-8") as file_in:
            values_dict = yaml.safe_load(file_in)

        values_dict["resc-database"]["hostOS"] = helm_values.operating_system
        values_dict["resc-database"]["database"]["pvc_path"] = helm_values.db_storage_path

        values_dict["resc-rabbitmq"]["filemountType"] = helm_values.operating_system
        values_dict["resc-rabbitmq"]["rabbitMQ"]["pvc_path"] = helm_values.rabbitmq_storage_path

        values_dict["resc-database"]["database"]["config"]["password"] = helm_values.db_password
        values_dict["resc-database-init"]["resc"]["config"]["dbPass"] = helm_values.db_password
        values_dict["resc-web-service"]["resc"]["config"]["dbPass"] = helm_values.db_password
        values_dict["resc-web-service-no-auth"]["resc"]["config"]["dbPass"] = helm_values.db_password

        values_dict["resc-vcs-instances"]["vcsInstances"] = prepare_vcs_instances_for_helm_values(
            helm_values=helm_values)

        with open(output_values_yaml_file, "w", encoding="utf-8") as file_out:
            yaml.dump(values_dict, file_out)
        output_values_yaml_file_path = os.path.abspath(output_values_yaml_file)
        if os.path.exists(output_values_yaml_file_path):
            logging.info(f"Helm values yaml file has been successfully generated at {output_values_yaml_file_path}")
            logging.info(f"You can proceed with deployment or you can refer to this link "
                         f"to make any customizations in helm values yaml file: {helm_deployment_help_link}")
            output_file_generated = True

    except FileNotFoundError:
        logging.error(f"Aborting the program! {input_values_yaml_file} file was not found")
        sys.exit(1)
    except KeyError as error:
        logging.error(f"Aborting the program! {error} was missing in {input_values_yaml_file}")
        sys.exit(1)
    return output_file_generated


def get_scheme_host_port_from_url(url: str):
    """
        Get scheme, host, port from url
    :param url:
        url of VCS instance
    :return: str, str, str
        Returns scheme, host, port
    """
    output = urlparse(url)
    if output.port:
        port = str(output.port)
    else:
        port = "443"
    return output.scheme, output.hostname, port


def get_vcs_instance_question_answers() -> List[VcsInstance]:
    """
        Get VCS instance related question answers
    :return: List[VcsInstance]
        Returns list of VCS instances
    """
    vcs_instance_answers = questions.ask_user_to_select_vcs_instance()

    if not vcs_instance_answers:
        logging.error("Aborting the program! No VCS instance was selected")
        sys.exit(1)

    vcs_instances: List[VcsInstance] = []

    for vcs in vcs_instance_answers:
        vcs_instance_info = questions.ask_vcs_instance_details(vcs_type=vcs)
        scheme, host, port = get_scheme_host_port_from_url(vcs_instance_info["url"])
        if vcs == "GitHub":
            default_github_accounts = f"{vcs_instance_info['username']}, kubernetes, docker"
            github_accounts = questions.ask_which_github_accounts_to_scan(
                default_github_accounts=default_github_accounts)
            github_account_list = [account.strip() for account in github_accounts.split(",")]
            vcs_instance = VcsInstance(
                provider_type="GITHUB_PUBLIC",
                scheme=scheme,
                host=host,
                port=port,
                username=vcs_instance_info["username"],
                password=vcs_instance_info["token"],
                organization=vcs_instance_info["organization"],
                scope=github_account_list
            )
        if vcs == "Azure Devops":
            vcs_instance = VcsInstance(
                provider_type="AZURE_DEVOPS",
                scheme=scheme,
                host=host,
                port=port,
                username=vcs_instance_info["username"],
                password=vcs_instance_info["token"],
                organization=vcs_instance_info["organization"],
                scope=[]
            )
        if vcs == "Bitbucket":
            vcs_instance = VcsInstance(
                provider_type="BITBUCKET",
                scheme=scheme,
                host=host,
                port=port,
                username=vcs_instance_info["username"],
                password=vcs_instance_info["token"],
                organization=vcs_instance_info["organization"],
                scope=[]
            )
        vcs_instances.append(vcs_instance)
    return vcs_instances


def download_rule_toml_file(url: str, file: str) -> bool:
    """
        Download rule toml file
    :param url:
        url of the file to download
    :param file:
        path of the downloaded file
    :return: bool
        Returns true if rule downloaded successfully else returns false
    """
    downloaded = False
    response = requests.get(url, timeout=100, verify=True)
    with open(file, "wb") as output:
        output.write(response.content)
    if os.path.exists(file) and os.path.getsize(file) > 0:
        downloaded = True
        logging.debug(f"{file} successfully downloaded")
    else:
        logging.error("Unable to download the rule file")
    return downloaded


def run_deployment_as_per_user_confirmation():
    """
        Run deployment as per user confirmation
    """
    run_deployment_confirm_msg = "Do you want to run deployment?"
    run_deployment_confirm = questions.ask_user_confirmation(msg=run_deployment_confirm_msg)
    if run_deployment_confirm is True:
        run_deployment()
    else:
        logging.info("Skipping deployment...")


def run_deployment():
    """
        Runs a helm deployment
    :return: bool
        Returns true if deployment successful else returns false
    """
    deployment_status = False

    rule_file_downloaded = download_rule_toml_file(url=constants.RULE_FILE_URL, file=constants.RULE_FILE)

    add_helm_repository()
    update_helm_repository()

    if rule_file_downloaded:
        namespace_created = create_namespace_if_not_exists(namespace_name=constants.NAMESPACE)

    if namespace_created:
        # Check if release is already installed
        output = subprocess.run(["helm", "list", "-n", constants.NAMESPACE], capture_output=True, text=True, check=True)
        # Check if same chart version is already installed
        version_installed = is_chart_version_already_installed()
        if constants.RELEASE_NAME in output.stdout and version_installed is True:
            run_upgrade_confirm_msg = f"Release {constants.RELEASE_NAME} is already installed in " \
                                      f"{constants.NAMESPACE} namespace. Do you want to upgrade the release?"
            run_upgrade_confirm = questions.ask_user_confirmation(msg=run_upgrade_confirm_msg)
            if run_upgrade_confirm is True:
                deployment_status = install_or_upgrade_helm_release(action="upgrade")
            else:
                logging.info("Skipping deployment...")

        else:
            deployment_status = install_or_upgrade_helm_release(action="install")
    return deployment_status
